import java.util.Map;
import java.util.TreeMap;

/**
 * Shutdown hook that prints summary of executed instructions
 * 
 * @author Kuba Madej
 * 
 */
public class ShutdownHook extends Thread {
	/**
	 * Delimiter between instruction name and executions count
	 */
	private static final String SPACER = "    ";

	/**
	 * Static initializer that registers this hook
	 */
	static {
		Runtime.getRuntime().addShutdownHook(new ShutdownHook());
	}

	/**
	 * Map that holds instructions count
	 */
	private final static Map<String, Integer> instructionCounts = new TreeMap<String, Integer>();

	/**
	 * Increments executions count of the given instruction
	 * 
	 * @param instructionName
	 *            instruction name
	 */
	public static void incrementInvocationsCount(final String instructionName) {
		final String upperCaseInstructionName = instructionName.toUpperCase();
		final Integer currentCount = instructionCounts
				.get(upperCaseInstructionName);
		if (currentCount == null) {
			instructionCounts.put(upperCaseInstructionName, 1);
		} else {
			instructionCounts.put(upperCaseInstructionName, currentCount + 1);
		}
	}

	/**
	 * Performs the actual summary print
	 */
	public void run() {
		for (Map.Entry<String, Integer> entry : instructionCounts.entrySet()) {
			System.out.print(entry.getKey());
			System.out.print(SPACER);
			System.out.println(entry.getValue());
		}
	}
}
