import java.util.Map;

/**
 * Statement implementation which assigns a given value to a certain variable
 *
 * @author Kuba Madej
 *
 */
public class AssignmentStatement implements Statement {
	/**
	 * Null expression
	 */
	private static final String NULL_EXPRESSION = "NULL";

	/**
	 * Numeric constant pattern
	 */
	private static final String NUMERIC_CONSTANT_PATTERN = "\\d+";

	/**
	 * Interpreter's heap
	 */
	private int[] heap;

	/**
	 * Variables map
	 */
	private Map<String, Integer> variables;

	/**
	 * L-value variable name
	 */
	private String lValueVariableName;

	/**
	 * R-value variable name, or numeric literal, or NULL
	 */
	private String rValueVariableName;

	/**
	 * Array of field dereferences applied to l-value variable
	 */
	private int[] lValueDereferencePath;

	/**
	 * Array of field dereferences applied to r-value variable
	 */
	private int[] rValueDereferencePath;

	/**
	 * Constructs new instance of this statement
	 *
	 * @param heap
	 *            interpreter's heap
	 * @param variables
	 *            variables map
	 * @param valueVariableName
	 *            l-value variable name
	 * @param valueVariableName2
	 *            r-value variable name, or numeric literal, or NULL
	 * @param valueDereferencePath
	 *            array of field dereferences applied to l-value variable
	 * @param valueDereferencePath2
	 *            array of field dereferences applied to r-value variable
	 */
	public AssignmentStatement(int[] heap, Map<String, Integer> variables,
			String valueVariableName, int[] valueDereferencePath,
			String valueVariableName2, int[] valueDereferencePath2) {
		this.heap = heap;
		this.variables = variables;
		lValueVariableName = valueVariableName;
		rValueVariableName = valueVariableName2;
		lValueDereferencePath = valueDereferencePath;
		rValueDereferencePath = valueDereferencePath2;
	}

	/**
	 *
	 * @see Statement#execute()
	 */
	public void execute() {
		int rValue;
		if (rValueVariableName.matches(NUMERIC_CONSTANT_PATTERN))
			rValue = Integer.parseInt(rValueVariableName);
		else if (NULL_EXPRESSION.equals(rValueVariableName))
			rValue = 0;
		else {
			rValue = variables.get(rValueVariableName);
			for (int i = 0; i < rValueDereferencePath.length; i++) {
				rValue = rValue + rValueDereferencePath[i];
				rValue = heap[rValue];
			}
		}

		if (lValueDereferencePath.length == 0)
			variables.put(lValueVariableName, rValue);
		else {
			int lValueIndex = variables.get(lValueVariableName);
			for (int i = 0; i < lValueDereferencePath.length; i++) {
				lValueIndex = lValueIndex + lValueDereferencePath[i];
				if (i < lValueDereferencePath.length - 1)
					lValueIndex = heap[lValueIndex];
			}

			heap[lValueIndex] = rValue;
		}
	}

}
